// code basé sur B. Frey, Datavisualization, 2008
FloatTable data; 
float dataMin, dataMax; 

float coinGX, coinGY; 
float coinDX, coinDY; 

int yearMin, yearMax; 
int[] years; 
PFont font; 

int rowCount;
int columnCount; 
int currentColumn = 0; 

int yearInterval = 5;
int depenseInterval = 1000;


float[] tabLeft, tabRight;  
float tabTop, tabBottom;
float tabPad = 10;

void setup() { 
  size(720, 405); 
  font = createFont("Arial", 14);
  textFont (font);
  data = new FloatTable("tab_depense_eleve.tsv"); 
  years = int(data.getRowNames()); 
  yearMin = years[0]; 
  yearMax = years[years.length - 1]; 

  dataMin = 0; 
  dataMax = data.getTableMax(); 
  rowCount = data.getRowCount();
    columnCount = data.getColumnCount();

  // coin en haut à gauche  et en bas à droite
  coinGX = 30; 
  coinDX = width - coinGX; 
  coinGY = 40; 
  coinDY = height - coinGY; 
  smooth(); 
}


void draw() { 
  background(224); 
  
  // champs des données 
  fill(255); 
  rectMode(CORNERS); 
  noStroke(); 
  rect(coinGX, coinGY, coinDX, coinDY); 
  
  // dessine le titre des données actuels
 drawTitle();

  // dessine les données 
drawDataPoints(currentColumn); 
drawYearLabels();
drawDepenseLabels();
drawTitleTabs();
if (mousePressed) saveFrame("####.png");
} 


// methodes pour dessiner les points
void drawDataPoints(int col) { 
  int rowCount = data.getRowCount(); 
  for (int row = 0; row < rowCount; row++) { 
    if (data.isValid(row, col)) { 
      float value = data.getFloat(row, col); 
      float x = map(years[row], yearMin, yearMax, coinGX, coinDX); 
      float y = map(value, dataMin, dataMax, coinDY, coinGY); 
      strokeWeight(5); 
      stroke(#5679C1); 
      point(x, y); 
    } 
  } 
} 

void drawTitle() {
  fill(0);
  textSize(20);
  textAlign(LEFT);
  String title = data.getColumnName(currentColumn);
  text(title, coinGX, coinGY - 10);
}

void drawYearLabels() {
  fill(0);
  textSize(10);
  textAlign(CENTER, TOP);
  

  stroke(224);
  strokeWeight(1);
  
  for (int row = 0; row < rowCount; row++) {
    if (years[row] % yearInterval == 0) {
      float x = map(years[row], yearMin, yearMax, coinGX, coinDX);
      text(years[row], x, coinDY + 5);
    }
  }
}

      
void drawDepenseLabels() {
  fill(0);
  textSize(10);
  textAlign(RIGHT, BOTTOM);

  stroke(224);
  strokeWeight(1);
  
  for (float i = dataMin; i < dataMax; i+=depenseInterval) {
      float y = map(i, dataMin, dataMax,  coinDY, coinGY);
      text(ceil(i),  coinGY -10, y);
  }
}

void drawTitleTabs() {
  rectMode(CORNERS);
  noStroke();
  textSize(20);
  textAlign(LEFT);

 //lors de la premier utilisation de la methode, allouer la memoire pour
 // les listes qui stockent les valeurs de gauche et droite des tabs
  if (tabLeft == null) {
    tabLeft = new float[columnCount];
    tabRight = new float[columnCount];
 
  }
  //commence avec le tab à gauche, runningX va sauter ensuite au bord droite du tab
  float runningX = coinGX; 
  // calcule haut en bas du tab
  tabTop = coinGY - textAscent() - 15;
  tabBottom = coinGY;
  
  // dessine un tab après l'autre
  for (int col = 0; col < columnCount; col++) {
    String title = data.getColumnName(col);// stocke le nom de la colonne dans un variable locale
    tabLeft[col] = runningX; // la position gauche de l'actuel colonne
    float titleWidth = textWidth(title);// calcule la position droite du tab
    tabRight[col] = tabLeft[col] + tabPad + titleWidth + tabPad; 

   //  si la colonne est  selectionné (cf mousePressed) rempli blanc sinon gris 
if (col == currentColumn)fill( 255);
 else fill (224);
 rect(tabLeft[col], tabTop, tabRight[col], tabBottom);
if (col == currentColumn) fill(50);
  else fill(124);
  text(title, runningX + tabPad, coinGY - 10);
    runningX = tabRight[col];
   }  
}

void mousePressed() {
  if (mouseY > tabTop && mouseY < tabBottom) {
    for (int col = 0; col < columnCount; col++) {
      if (mouseX > tabLeft[col] && mouseX < tabRight[col]) {
  currentColumn = col;
      }
    }
  }
}

